#!/usr/bin/env python2.3
"""
Generate a sequence set from given models.

Usage: GenDataSet.py [options] <input_models_file.smo> <output_sequences_file.sqd>
Options:
    -h          --help      Print this help message.
    -l number   --length    Length of sequences to generate.
    -c list     --counts    How many sequences per model.

Example: > python GenDataSet.py -l 7 -c "2 3 4 5 6" spo_best.smo generated.sqd

Notes:
 * Requires the ghmm module.
 * If the sequence file already exists, the new sequence set is appended.
"""

import sys
import getopt
import ghmm
import ghmmwrapper

def GenerateSequences(models, seq_per_model, seq_len):
    seqs = None
    for i in range(len(models)):
        count = 0
        while count < seq_per_model[i]:
            emmission = models[i].sampleSingle(seq_len)
            if len(emmission) < seq_len:
                continue
            elif emmission[seq_len - 1] < 9990:
                continue
            else:
                if seqs == None:
                    seqs = emmission.sequenceSet()
                else:
                    seqs.merge(emmission)

                # Sets the model number for the sequence.
                ghmmwrapper.set_sequence_d_label(seqs.cseq, len(seqs) - 1, i)
                count += 1
    return seqs



class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

def main(argv=None):
    # Default values
    seq_len = 12
    seq_per_model = [20]

    argv = sys.argv
    
    try:
        try:
            opts, args = getopt.getopt(argv[1:], 'hl:c:', ['help', 'length', 'counts'])
        except getopt.error, msg:
             raise Usage(msg)

        # process options
        for o, a in opts:
            if o in ('-h', '--help'):
                print __doc__
                return 0
            if o in ('-l', '--length'):
                # Add 1 for the final state.
                seq_len = int(a) + 1
            if o in ('-c', '--counts'):
                seq_per_model = map(int, a.split())

        if len(args) == 0:
            raise Usage('No models file specified')
        elif len(args) == 1:
            raise Usage('No sequences file specified')
        elif len(args) >= 3:
            raise Usage('Too many arguments')

        smo_file, sqd_file = args
        models = ghmm.HMMOpen.all(smo_file)
        if len(models) == 0:
            raise Usage('No models found')
        while len(seq_per_model) < len(models):
            seq_per_model.append(seq_per_model[-1])
        seqs = GenerateSequences(models, seq_per_model, seq_len)
        seqs.write(sqd_file)

    except Usage, err:
        print >>sys.stderr, err.msg
        print >>sys.stderr, "for help use --help"
        return 2

if __name__ == "__main__":
    sys.exit(main())

